<?php

namespace Assetdocs\Department\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Department\Http\Requests\DepartmentAdminWebRequest;
use Assetdocs\Department\Interfaces\DepartmentRepositoryInterface;
use Assetdocs\Department\Models\Department;

/**
 * Admin web controller class.
 */
class DepartmentAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize department controller.
     *
     * @param type DepartmentRepositoryInterface $department
     *
     * @return type
     */
    public function __construct(DepartmentRepositoryInterface $department)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $department;
        parent::__construct();
    }

    /**
     * Display a list of department.
     *
     * @return Response
     */
    public function index(DepartmentAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $departments  = $this->repository
            ->pushCriteria(new \Assetdocs\Department\Repositories\Criteria\DepartmentAdminCriteria())
                ->setPresenter('\\Assetdocs\\Department\\Repositories\\Presenter\\DepartmentListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $departments['recordsTotal']    = $departments['meta']['pagination']['total'];
            $departments['recordsFiltered'] = $departments['meta']['pagination']['total'];
            $departments['request']         = $request->all();
            return response()->json($departments, 200);

        }

        $this   ->theme->prependTitle(trans('department::department.names').' :: ');
        return $this->theme->of('department::admin.department.index')->render();
    }

    /**
     * Display department.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return Response
     */
    public function show(DepartmentAdminWebRequest $request, Department $department)
    {
        if (!$department->exists) {
            return response()->view('department::admin.department.new', compact('department'));
        }

        Form::populate($department);
        return response()->view('department::admin.department.show', compact('department'));
    }

    /**
     * Show the form for creating a new department.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DepartmentAdminWebRequest $request)
    {

        $department = $this->repository->newInstance([]);

        Form::populate($department);

        return response()->view('department::admin.department.create', compact('department'));

    }

    /**
     * Create new department.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DepartmentAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $department          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('department::department.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/department/department/'.$department->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show department for editing.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return Response
     */
    public function edit(DepartmentAdminWebRequest $request, Department $department)
    {
        Form::populate($department);
        return  response()->view('department::admin.department.edit', compact('department'));
    }

    /**
     * Update the department.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return Response
     */
    public function update(DepartmentAdminWebRequest $request, Department $department)
    {
        try {

            $attributes = $request->all();

            $department->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('department::department.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/department/department/'.$department->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/department/department/'.$department->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the department.
     *
     * @param Model   $department
     *
     * @return Response
     */
    public function destroy(DepartmentAdminWebRequest $request, Department $department)
    {

        try {

            $t = $department->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('department::department.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/department/department/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/department/department/'.$department->getRouteKey()),
            ], 400);
        }
    }
}
